// FindSymbolDlg.cpp : implementation file
//

#include "stdafx.h"
#include "LibDump.h"
#include "FindSymbolDlg.h"
#include "MainFrm.h"
#include "DirDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CLibDumpApp theApp;

LPCTSTR g_lpszFileType = _T("FilesOfType");
LPCTSTR g_lpszFindWhatSection = _T("FindWhat");
LPCTSTR g_lpszFindInFolderSection = _T("FindInFolder");
LPCTSTR g_lpszEntry	= _T("Find");
LPCTSTR g_lpszInitialFindWhat = _T("Initial_Find_What");
LPCTSTR g_lpszInitialFindFolder = _T("Initial_Find_Folder");
LPCTSTR g_lpszMatchWholeWordOnly = _T("Match_Whole_Word_Only");
LPCTSTR g_lpszMatchCase = _T("Match_Case");
LPCTSTR g_lpszLookInSubFolders = _T("Look_In_Sub_Folders");

const int g_nValuesToStore = 16;
/////////////////////////////////////////////////////////////////////////////
// CFindSymbolDlg dialog

CFindSymbolDlg::CFindSymbolDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFindSymbolDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFindSymbolDlg)
	m_bLookInSubFolders = FALSE;
	m_bMatchCase = FALSE;
	m_bMatchWholeWordOnly = FALSE;
	m_nFilesOfType = 0;
	//}}AFX_DATA_INIT
}

void CFindSymbolDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFindSymbolDlg)
	DDX_Control(pDX, IDC_FILE_TYPE, m_cbFilesOfType);
	DDX_Control(pDX, IDC_COMBO2, m_cbInFolder);
	DDX_Control(pDX, IDC_COMBO1, m_cbFindWhat);
	DDX_Check(pDX, IDC_CHECK2, m_bLookInSubFolders);
	DDX_Check(pDX, IDC_CHECK3, m_bMatchCase);
	DDX_Check(pDX, IDC_CHECK1, m_bMatchWholeWordOnly);
	DDX_CBIndex(pDX, IDC_FILE_TYPE, m_nFilesOfType);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CFindSymbolDlg, CDialog)
	//{{AFX_MSG_MAP(CFindSymbolDlg)
	ON_BN_CLICKED(IDC_BUTTON1, OnChooseFolder)
	ON_CBN_SELCHANGE(IDC_FILE_TYPE, OnSelchangeFileType)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFindSymbolDlg message handlers

BOOL CFindSymbolDlg::OnInitDialog() 
{
	CWaitCursor wait;

	CDialog::OnInitDialog();
	
	CString sBuff, sValue;
	for(int i = 1; i <= g_nValuesToStore; i++)
	{
		sBuff.Format(
			_T("%s_%d"), g_lpszEntry, i
			);
		sValue = theApp.GetProfileString(g_lpszFindWhatSection, sBuff);
		if(!sValue.IsEmpty())
			m_cbFindWhat.AddString(sValue);
	}
	sValue.Empty();
	
	try
	{
		sValue = GetDumpLibView()->GetRichEditCtrl().GetSelText();
	}
	catch(...)
	{
		// Scenario of large seltext
	}
	if(sValue.IsEmpty() || (sValue.FindOneOf(_T("\n\r")) != -1))
		sValue = theApp.GetProfileString(g_lpszFindWhatSection, g_lpszInitialFindWhat);
	m_cbFindWhat.SetWindowText(sValue);
	
	// Files of type
	m_nFilesOfType = theApp.GetProfileInt(
		g_lpszFindInFolderSection, g_lpszFileType, 0
		);
	m_cbFilesOfType.SetCurSel(m_nFilesOfType);
	OnSelchangeFileType();

	for(int j = 1; j <= g_nValuesToStore; j++)
	{
		sBuff.Format(
			_T("%s_%d"), g_lpszEntry, j
			);
		sValue = theApp.GetProfileString(g_lpszFindInFolderSection, sBuff);
		sValue.TrimRight(); sValue.TrimLeft();
		if(!sValue.IsEmpty())
			m_cbInFolder.AddString(sValue);
	}
	sValue = theApp.GetProfileString(g_lpszFindInFolderSection, g_lpszInitialFindFolder);
	m_cbInFolder.SetWindowText(sValue);

	m_bMatchWholeWordOnly = theApp.GetProfileInt(
		g_lpszFindInFolderSection, g_lpszMatchWholeWordOnly, 0
		);
	m_bMatchCase = theApp.GetProfileInt(
		g_lpszFindInFolderSection, g_lpszMatchCase, 0
		);
	m_bLookInSubFolders = theApp.GetProfileInt(
		g_lpszFindInFolderSection, g_lpszLookInSubFolders, 1
		);
	
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
}

void CFindSymbolDlg::OnOK() 
{
	CStringEx sBuff, sLBText, sFolderToSearch;

	UpdateData();
	
	// FindWhat
	m_cbFindWhat.GetWindowText(m_strSymbolToSearch);
	if(m_strSymbolToSearch.IsEmpty())
	{
		AfxMessageBox(_T("<Find what> is a required field."), MB_ICONINFORMATION);
		m_cbFindWhat.SetFocus();
		return;
	}
			
	// FindInFolder
	m_cbInFolder.GetWindowText(m_strFolderToSearch);
	m_strFolderToSearch.TrimRight();
	m_strFolderToSearch.TrimLeft();
	if(m_strFolderToSearch.IsEmpty())
	{
		AfxMessageBox(_T("<In folder> is a required field."), MB_ICONINFORMATION);
		m_cbInFolder.SetFocus();
		return;
	}

	// Store the original current directory
	::GetCurrentDirectory(
		_MAX_PATH, m_strCurrentDir.GetBuffer(_MAX_PATH+1)
		);
	m_strCurrentDir.ReleaseBuffer();
	if(!::SetCurrentDirectory(m_strFolderToSearch))
	{
		sBuff.Format(
			_T("The folder <%s> does not exist."), (LPCTSTR)m_strFolderToSearch
			);
		AfxMessageBox(sBuff, MB_ICONINFORMATION);
		m_cbInFolder.SetFocus();
		return;
	}

	// FindWhat
	int nCount = m_cbFindWhat.GetCount();
	BOOL bDuplicateExists = FALSE;
	for(int n = 1; n <= nCount; n++)
	{
		m_cbFindWhat.GetLBText(n-1, sLBText);
		if(sLBText == m_strSymbolToSearch)
		{
			bDuplicateExists = TRUE;
			break;
		}
	}
	if(!bDuplicateExists)
		m_cbFindWhat.InsertString(0, m_strSymbolToSearch);

	nCount = m_cbFindWhat.GetCount();
	if(nCount > g_nValuesToStore)
		nCount = m_cbFindWhat.DeleteString(nCount-1); // Store upto 16 values only 
	for(n = 1; n <= nCount; n++)
	{
		m_cbFindWhat.GetLBText(n-1, sLBText);
		sBuff.Format(
			_T("%s_%d"), g_lpszEntry, n
			);
		theApp.WriteProfileString(g_lpszFindWhatSection, sBuff, sLBText);
	}
	theApp.WriteProfileString(
		g_lpszFindWhatSection, g_lpszInitialFindWhat, m_strSymbolToSearch
		);

	// Files of type
	theApp.WriteProfileInt(
		g_lpszFindInFolderSection, g_lpszFileType, m_nFilesOfType
		);

	// FindInFolder
	nCount = m_cbInFolder.GetCount();
	bDuplicateExists = FALSE;
	for(n = 1; n <= m_cbInFolder.GetCount(); n++)
	{
		m_cbInFolder.GetLBText(n-1, sLBText);
		sBuff = sLBText;
		sFolderToSearch = m_strFolderToSearch;
		sBuff.MakeUpper();
		sFolderToSearch.MakeUpper();
		if(sBuff == sFolderToSearch)
		{
			bDuplicateExists = TRUE;
			break;
		}
	}
	
	if(!bDuplicateExists)
		m_cbInFolder.InsertString(0, m_strFolderToSearch);
	
	nCount = m_cbInFolder.GetCount();
	if(nCount > g_nValuesToStore)
		nCount = m_cbInFolder.DeleteString(nCount);	// Store upto 16 values only 
	for(n = 1; n <= nCount; n++)
	{
		m_cbInFolder.GetLBText(n-1, sLBText);
		sBuff.Format(
			_T("%s_%d"), g_lpszEntry, n
			);
		theApp.WriteProfileString(g_lpszFindInFolderSection, sBuff, sLBText);
	}

	theApp.WriteProfileString(
		g_lpszFindInFolderSection, g_lpszInitialFindFolder, m_strFolderToSearch
		);
	theApp.WriteProfileInt(
		g_lpszFindInFolderSection, g_lpszMatchWholeWordOnly, m_bMatchWholeWordOnly
		);
	theApp.WriteProfileInt(
		g_lpszFindInFolderSection, g_lpszMatchCase, m_bMatchCase
		);
	theApp.WriteProfileInt(
		g_lpszFindInFolderSection, g_lpszLookInSubFolders, m_bLookInSubFolders
		);
	
	TRACE(_T("SymbolToSearchFor: %s\n"), (LPCTSTR)m_strSymbolToSearch);
	TRACE(_T("FolderToSearchFor: %s\n"), (LPCTSTR)m_strFolderToSearch);

	CDialog::OnOK();
}

void CFindSymbolDlg::OnChooseFolder() 
{
	CWaitCursor wait;

	CDirDialog dlg(this);
	dlg.m_strTitle = "Select the folder where you want to begin the search.";
	m_cbInFolder.GetWindowText(dlg.m_strSelDir);
	if(dlg.DoBrowse())
	{
		m_strFolderToSearch = dlg.m_strPath;
		m_cbInFolder.SetWindowText(m_strFolderToSearch);
	}
}

CLibDumpView* CFindSymbolDlg::GetDumpLibView()
{
	return (CLibDumpView*)((CMainFrame*)AfxGetMainWnd())->GetActiveView();
}

void CFindSymbolDlg::OnSelchangeFileType() 
{
	UpdateData();
	if(m_nFilesOfType == 1)
	{
		GetDlgItem(IDC_CHECK1)->SetWindowText(
			_T("Match &whole word only")
			);
	}
	else
	{
		GetDlgItem(IDC_CHECK1)->SetWindowText(
			_T("Match &whole symbol only")
			);
	}
}