// LibDumpDlg.cpp : implementation file
//

#include "stdafx.h"
#include "LibDump.h"
#include "LibDumpDlg.h"
#include "LibDumpView.h"
#include "LibSymbolInfo.h"
#include "TypeLib.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CLibDumpApp theApp;

LPCTSTR g_szGeneral = _T("General");
LPCTSTR g_szFormat = _T("COFF Format");
/////////////////////////////////////////////////////////////////////////////
// CLibDumpDlg dialog

CLibDumpDlg::CLibDumpDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLibDumpDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLibDumpDlg)
	m_strLibPathName = _T("");
	m_bCOFFFormat = FALSE;
	//}}AFX_DATA_INIT
}

void CLibDumpDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLibDumpDlg)
	DDX_Text(pDX, IDC_LIB_PATH_NAME, m_strLibPathName);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLibDumpDlg, CDialog)
	//{{AFX_MSG_MAP(CLibDumpDlg)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLibDumpDlg message handlers

BOOL CLibDumpDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_pFrame = (CMainFrame*)AfxGetMainWnd();
	m_pWndStatusBar = &m_pFrame->m_wndStatusBar;

	m_bCOFFFormat = theApp.GetProfileInt(g_szGeneral, g_szFormat, 1);
	if(m_bCOFFFormat)
		((CButton*)GetDlgItem(IDC_COFF))->SetCheck(TRUE);
	else
		((CButton*)GetDlgItem(IDC_TYPE_LIB))->SetCheck(TRUE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CLibDumpDlg::OnOK() 
{
	CWaitCursor wait;

	UpdateData();

	m_strLibPathName.TrimRight();
	m_strLibPathName.TrimLeft();
	if(m_strLibPathName.IsEmpty())
	{
		AfxMessageBox(
			_T("<File Name> is a required field."), MB_ICONINFORMATION
			);
		GetDlgItem(IDC_LIB_PATH_NAME)->SetFocus();
		return;
	}

	int nPos = m_strLibPathName.ReverseFind(_T('\\'));
	if(nPos != -1)
		m_strLibFileName = m_strLibPathName.Mid(nPos+1);
		
	TCHAR lpszTempPath[_MAX_PATH];
	TCHAR lpszTempFileName[_MAX_PATH];
	DWORD dwSize = ::GetTempPath(_MAX_PATH, lpszTempPath);
	ASSERT(dwSize);
	UINT nRet = ::GetTempFileName(lpszTempPath, m_strLibFileName, 0, lpszTempFileName);
	ASSERT(nRet);
  	CString sBuff;
	CStdioFile file;
	CFileException fileException;
	if(!file.Open(lpszTempFileName, CFile::typeText | CFile::modeCreate |  
		CFile::modeWrite, &fileException))
	{
		sBuff.Format(_T("File Path Name: <%s> "), (LPCTSTR)lpszTempFileName);
		sBuff += CHelpers::GetFileExceptionError(fileException.m_cause);
		AfxMessageBox(sBuff, MB_ICONERROR);
		GetDlgItem(IDC_LIB_PATH_NAME)->SetFocus();
		return;
	}

	m_pWndStatusBar->SetPaneText(0, _T("Processing..."));

	m_bCOFFFormat = ((CButton*)GetDlgItem(IDC_COFF))->GetCheck();
	BOOL bRet = FALSE;
	if(m_bCOFFFormat)
		bRet = DumpSymbols(&file, sBuff);
	else
		bRet = DumpTypeLibrary(&file, sBuff);
	if(!bRet)
	{
		AfxMessageBox(sBuff, MB_ICONERROR);
		GetDlgItem(IDC_LIB_PATH_NAME)->SetFocus();
		sBuff.LoadString(AFX_IDS_IDLEMESSAGE);
		m_pWndStatusBar->SetPaneText(0, sBuff);
	}
	
	if(file.m_pStream != NULL)
		file.Close();
	
	if(bRet)
		bRet = m_pFrame->GetActiveDocument()->OnOpenDocument(lpszTempFileName);
	
	CFile::Remove(lpszTempFileName);

	if(bRet)
	{
		theApp.WriteProfileInt(g_szGeneral, g_szFormat, m_bCOFFFormat);
		CDialog::OnOK();
	}
}

void CLibDumpDlg::OnBrowse()
{
	CWaitCursor wait;

	UpdateData();

	m_bCOFFFormat = ((CButton*)GetDlgItem(IDC_COFF))->GetCheck();
	CString sFilter;
	if(m_bCOFFFormat)
	{
		sFilter = _T("Library Files (*.lib)|*.lib|");
		sFilter +=_T("All Files (*.*)|*.*|");
	}
	else
	{
		sFilter =_T("Type Library Files (*.tlb;*.olb;*.dll;*.ocx;*.exe)|");
		sFilter +=_T("*.tlb;*.olb;*.dll;*.ocx;*.exe|");
		sFilter +=_T("All Files (*.*)|*.*|");
	}
	CFileDialog dlg(
		TRUE, NULL, NULL, OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST,
        sFilter, this
		);
	dlg.m_ofn.lpstrInitialDir = (LPCTSTR)m_strLibPathName;
	if(dlg.DoModal() == IDOK)
	{
		m_strLibPathName = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

BOOL CLibDumpDlg::DumpSymbols(CStdioFile* pFile, CString& sLastError)
{
	return ((CLibDumpView*)m_pFrame->GetActiveView())->DumpLibrary(
		pFile, m_strLibPathName, sLastError
		);
}

BOOL CLibDumpDlg::DumpTypeLibrary(CStdioFile* pFile, CString& sLastError)
{
	BOOL bRet = FALSE;

	ASSERT(pFile && pFile->m_pStream);
	sLastError.Empty();

	try
	{
		HRESULT hr = ((CLibDumpView*)m_pFrame->GetActiveView())->DumpTypeLibrary(
			pFile, m_strLibPathName
			);
		if(FAILED(hr))
			_com_issue_error(hr);
		bRet = TRUE;
	}
	catch(const _com_error& e)
	{
		sLastError = CHelpers::GetComError(e);
	}
	catch(...)
	{
		sLastError = _T("Errors occurred.");
	}

	return bRet;
}