// LibDumpView.cpp : implementation of the CLibDumpView class
//

#include "stdafx.h"
#include "LibDump.h"

#include "LibDumpDoc.h"
#include "LibDumpView.h"
#include "LibDumpDlg.h"
#include "FindSymbolDlg.h"
#include "TypeLib.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CLibDumpApp theApp;

LPCTSTR g_szResults = _T("Results");

extern LPCTSTR g_szGeneral;
LPCTSTR g_szWordWrap = _T("Word Wrap");
/////////////////////////////////////////////////////////////////////////////
// CLibDumpView

IMPLEMENT_DYNCREATE(CLibDumpView, CRichEditView)

BEGIN_MESSAGE_MAP(CLibDumpView, CRichEditView)
	//{{AFX_MSG_MAP(CLibDumpView)
	ON_COMMAND(ID_FILE_DUMP, OnFileDump)
	ON_COMMAND(ID_FILE_FIND_SYMBOLS, OnFileFindSymbols)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_WM_CREATE()
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_COMMAND(ID_EDIT_WORD_WRAP, OnEditWordWrap)
	ON_UPDATE_COMMAND_UI(ID_EDIT_WORD_WRAP, OnUpdateEditWordWrap)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_RIGHTCLICK_OPEN, OnRightclickOpen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLibDumpView construction/destruction

CLibDumpView::CLibDumpView() :
	m_bPrintSelection(FALSE),
	m_nNextSelCharToBePrint(-1),
	m_nFirstNonSelChar(-1),
	m_nTypeLibSymbolOccurences(0),
	m_bCOFF(FALSE),
	m_pEdit(NULL)
{
	m_nWordWrap = theApp.GetProfileInt(g_szGeneral, g_szWordWrap, WrapNone);
		
	LOGFONT logFont;
	memset(&logFont, 0, sizeof(logFont));
	logFont.lfHeight = -13;
	logFont.lfWeight = FW_NORMAL;
	_tcscpy(logFont.lfFaceName, _T("Courier"));
	if(!m_font.CreateFontIndirect(&logFont))
		TRACE(_T("Could Not create font.\n"));
}

CLibDumpView::~CLibDumpView()
{
	if(m_font.m_hObject != NULL)
	{
		m_font.DeleteObject();
		m_font.m_hObject = NULL;
	}
}

void CLibDumpView::OnInitialUpdate()
{
	CRichEditView::OnInitialUpdate();
	
	m_pWndStatusBar = &((CMainFrame*)AfxGetMainWnd())->m_wndStatusBar;
}

/////////////////////////////////////////////////////////////////////////////
// CLibDumpView diagnostics

#ifdef _DEBUG
void CLibDumpView::AssertValid() const
{
	CRichEditView::AssertValid();
}

void CLibDumpView::Dump(CDumpContext& dc) const
{
	CRichEditView::Dump(dc);
}

CLibDumpDoc* CLibDumpView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CLibDumpDoc)));
	return (CLibDumpDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLibDumpView message handlers

void CLibDumpView::OnFileDump() 
{
	CWaitCursor wait;

    CLibDumpDlg dlg(this);
	if(dlg.DoModal() == IDOK)
	{
		m_bCOFF = dlg.m_bCOFFFormat;
		m_strLibName = dlg.m_strLibFileName;
		CString sBuff;
		m_pEdit->SetModify(FALSE);
		sBuff.LoadString(AFX_IDS_IDLEMESSAGE);
		m_pWndStatusBar->SetPaneText(0, sBuff);
	}
}

void CLibDumpView::OnFileSaveAs() 
{
	CString sFilter;
	CString sTitle = m_strLibName;
	if(sTitle == g_szResults)
	{
		sTitle = _T("[Find in Files]");
		sFilter = _T("Output Files (*.txt)|*.txt|All Files (*.*)|*.*|");
	}
	else
	{
		int nPos = sTitle.Find(_T('.'));
		if(nPos != -1)
			sTitle = sTitle.Left(nPos);
		if(m_bCOFF)
			sFilter = _T("Library Info Files (*.txt)|*.txt|All Files (*.*)|*.*|");
		else
			sFilter = _T("Type Library Info Files (*.txt)|*.txt|All Files (*.*)|*.*|");
	}

	CFileDialog dlg(
		false, _T("txt"), sTitle, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		sFilter, this
		);
	if(dlg.DoModal() == IDOK)
	{
		CWaitCursor wait;

		CString sPathName = dlg.GetPathName();
		CLibDumpDoc* pDoc = (CLibDumpDoc*)GetDocument();
		ASSERT(pDoc != NULL);
		if(!pDoc->OnSaveDocument(sPathName))
			TRACE(_T("Error saving document: %s\n"), (LPCTSTR)sPathName);
	}
}

void CLibDumpView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pEdit != NULL && m_pEdit->GetTextLength());
}

void CLibDumpView::OnEditWordWrap() 
{
	if(m_nWordWrap == WrapNone)
	{
		m_nWordWrap = WrapToWindow;
		SendMessage(WM_HSCROLL, SB_LEFT);
	}
	else
		m_nWordWrap = WrapNone;
	WrapChanged();
	theApp.WriteProfileInt(g_szGeneral, g_szWordWrap, m_nWordWrap);
}

void CLibDumpView::OnUpdateEditWordWrap(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pEdit != NULL && m_pEdit->GetTextLength());
	pCmdUI->SetCheck(m_nWordWrap == WrapToWindow);
}

void CLibDumpView::OnFileFindSymbols() 
{
	CFindSymbolDlg dlg;
	
	if(dlg.DoModal() == IDOK)
	{
		m_strLibName = g_szResults;
			
		CStopDlg dlgStop;
		
		m_pWndStatusBar->SetPaneText(0, _T("Processing..."));

		m_bCOFF = (dlg.m_nFilesOfType == 0) ? 1 : 0;

		if(dlg.m_nFilesOfType)
			m_nTypeLibSymbolOccurences = 0;
		
		CString sFormat = _T("              Find what: %s\n");
				sFormat +=_T("          Files of type: %s\n");
				sFormat +=_T("              In folder: %s\n");
				if(m_bCOFF)
				{
					sFormat +=_T("Match whole symbol only: %d\n");
				}
				else
				{
					sFormat +=_T("  Match whole word only: %d\n");
				}
				sFormat +=_T("     Look in subfolders: %d\n");
				sFormat +=_T("             Match case: %d\n\n");
		CString sBuff;
		sBuff.Format(
			sFormat,
			(LPCTSTR)dlg.m_strSymbolToSearch,
			(LPCTSTR)(
				m_bCOFF ? _T("COFF formats") : _T("Type libraries")
				),
			(LPCTSTR)dlg.m_strFolderToSearch,
			dlg.m_bMatchWholeWordOnly,
			dlg.m_bLookInSubFolders,
			dlg.m_bMatchCase
			);
		m_pEdit->SetSel(0, -1);
		m_pEdit->ReplaceSel(sBuff);
		
		try
		{
			CLibSymbolInfo libInfo;
			TraverseDirectory(
				libInfo, dlg.m_strSymbolToSearch, dlg.m_bLookInSubFolders,
				dlg.m_bMatchCase, dlg.m_bMatchWholeWordOnly, &dlgStop,
				m_bCOFF
				);
			if(dlgStop.m_bStop)
				sBuff = "Canceled.";
			else
				sBuff.Format(
					_T("%d occurrence(s) have been found."),
					m_bCOFF ? libInfo.GetOccurences() : m_nTypeLibSymbolOccurences
					);
			m_pEdit->SetSel(-1, -1);
			m_pEdit->ReplaceSel(sBuff);
		}
		catch(...)
		{
			m_pEdit->SetSel(-1, -1);
			m_pEdit->ReplaceSel(_T("Errors occurred. Search aborted."));
		}
		int nLen = m_pEdit->GetTextLength();
		m_pEdit->SetSel(nLen, nLen);

		::MessageBeep(0);
		
		::SetCurrentDirectory(dlg.m_strCurrentDir);
		
		m_pEdit->SetModify(FALSE);

		sBuff.LoadString(AFX_IDS_IDLEMESSAGE);
		m_pWndStatusBar->SetPaneText(0, sBuff); 
	}
}

void CLibDumpView::GrepTypeLibSymbols(
	const CString& sLibFilePath, const CString& sSymbolToGrep,
	const BOOL& bMatchCase, const BOOL& bMatchWholeWord, CStopDlg* pDlg
	)
{
	ASSERT(pDlg != NULL);

	CString sText;
	BOOL bRet = FALSE;
	LPTYPELIB lpTypeLib = NULL;
	if(SUCCEEDED(CTypeLib::IsTypeLib(sLibFilePath, lpTypeLib)))
	{
		try
		{
			HRESULT hr = CTypeLib::GetTypeLibAsString(lpTypeLib, sText);
			if(FAILED(hr))
				_com_issue_error(hr);
			bRet = TRUE;
		}
		catch(...)
		{
			TRACE(_T("Errors occurred.\n"));
		}
	}

	if(bRet)
	{
		// Do a fast find and calculate the line #
		CStringEx sTextEx = sText;
		int nPos = -1;
		if(bMatchCase)
			nPos = sTextEx.Find(sSymbolToGrep);
		else
			nPos = sTextEx.FindNoCase(sSymbolToGrep);
		if(nPos != -1)
		{
			CString sBuff = sTextEx.Left(nPos);
			int nFind = sBuff.ReverseFind(_T('\n')), nLine = 0;
			if(nFind != -1)
			{
				sTextEx = sBuff.Mid(nFind+1) + sTextEx.Mid(nPos);
				
				// Calculate the line #
				sBuff = sBuff.Left(nFind+1);
				nPos = -1;
				do
				{
					nPos = sBuff.Find(_T("\n"), nPos+1);
					if(nPos != -1)
						nLine++;
				}
				while(nPos != -1);
			}
			sTextEx += _T("\n");
			CStringEx sSymbolInfo;
			int nBuff = 250;
			do
			{
				nPos = sTextEx.Find(_T("\n"));
				if(nPos != -1)
				{
					sSymbolInfo = sTextEx.Left(nPos);
					nLine++;
				}
				if(bMatchCase)
				{
					nFind = sSymbolInfo.Find(sSymbolToGrep);
				}
				else
				{
					nFind = sSymbolInfo.FindNoCase(sSymbolToGrep);
				}
				if(nFind != -1 && bMatchWholeWord)
				{
					if(!MatchWholeWord(nFind, sSymbolInfo, sSymbolToGrep))
					{
						nFind = -1;
					}
				}
				if(nFind != -1)
				{
					sSymbolInfo.TrimRight();
					sBuff.Format(
						_T("%s (%d): %s\r\n"),
						(LPCTSTR)sLibFilePath,
						nLine,
						(LPCTSTR)sSymbolInfo
						);
					m_pEdit->SetSel(-1, -1);
					m_pEdit->ReplaceSel(sBuff);
					m_nTypeLibSymbolOccurences++;
				}
				if(nPos != -1)
				{
					sTextEx = sTextEx.Mid(nPos+1);
				}
				
				if(nLine > nBuff)
				{
					nBuff += 250; // Pump at an interval of 250 lines
					pDlg->Pump();
					if(pDlg->m_bStop)
						break;
				}
			}
			while(nPos != -1);
		}
	}
}

void CLibDumpView::TraverseDirectory(CLibSymbolInfo& libInfo, const CString& sSymbolToGrep, 
	const BOOL& bLookInSubFolders, const BOOL& bMatchCase, const BOOL& bMatchWholeWord,
	CStopDlg* pDlg, const BOOL& bCOFF)
{
	pDlg->Pump();
	if(pDlg->m_bStop)
		return;
	
	BOOL bIsTypeLibExt = FALSE;
	CString sPattern, sCurrentDir, sLibFilePath, sBuff;
	if(bCOFF)
		sPattern = _T("*.lib");
	else
		sPattern = _T("*.*");
	LPTSTR lpszPattern = (LPTSTR)(LPCTSTR)sPattern;
	WIN32_FIND_DATA FindData;
	HANDLE hFind = ::FindFirstFile(lpszPattern, &FindData);
	BOOL bOK  = (hFind != INVALID_HANDLE_VALUE);
	while(bOK)
	{
		if(!(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) && !IsChildDir(&FindData))
		{
			::GetCurrentDirectory(_MAX_PATH, sCurrentDir.GetBuffer(_MAX_PATH+1));
			sCurrentDir.ReleaseBuffer();
			if(!bCOFF)
			{
				CStringEx sFileName = FindData.cFileName;
				bIsTypeLibExt = (
					sFileName.FindNoCase(_T(".tlb")) != -1 ||
					sFileName.FindNoCase(_T(".olb")) != -1 ||
					sFileName.FindNoCase(_T(".dll")) != -1 ||
					sFileName.FindNoCase(_T(".ocx")) != -1 ||
					sFileName.FindNoCase(_T(".exe")) != -1
					);
				if(!bIsTypeLibExt)
				{
					sBuff = _T("Folder being scanned: ");
					sBuff +=sCurrentDir;
				}
			}
			if(bCOFF || bIsTypeLibExt)
			{
				sLibFilePath = sCurrentDir;
				sLibFilePath +=_T("\\");
				sLibFilePath +=FindData.cFileName;
				sBuff = _T("File being scanned: ");
				sBuff +=sLibFilePath;
			}
			m_pWndStatusBar->SetPaneText(0, sBuff);
			
			if(bCOFF)
			{
				libInfo.GrepSymbols(
					(LPTSTR)(LPCTSTR)sLibFilePath, m_pEdit,
					sSymbolToGrep, bMatchCase, bMatchWholeWord
					);
			}
			else if(bIsTypeLibExt)
			{
				GrepTypeLibSymbols(
					sLibFilePath, sSymbolToGrep, bMatchCase, bMatchWholeWord, pDlg
					);
			}
		}
		
		pDlg->Pump();
		if(pDlg->m_bStop)
			return;

		bOK = ::FindNextFile(hFind, &FindData);
	}
   
	if(hFind != INVALID_HANDLE_VALUE)
		::FindClose(hFind);

	if(bLookInSubFolders)
	{
		lpszPattern = _T("*.*");

		// Get the first child directory
		hFind = FindFirstChildDir(lpszPattern, &FindData);
		bOK = (hFind != INVALID_HANDLE_VALUE);
		while(bOK)
		{
			pDlg->Pump();
			if(pDlg->m_bStop)
				return;

			// Change into the child directory
			if(::SetCurrentDirectory(FindData.cFileName))
			{
				::GetCurrentDirectory(_MAX_PATH, sCurrentDir.GetBuffer(_MAX_PATH+1));
				sCurrentDir.ReleaseBuffer();
				sBuff = _T("Folder being scanned: ");
				sBuff +=sCurrentDir;
				m_pWndStatusBar->SetPaneText(0, sBuff);
							
				// Perform the recursive walk into the child
				// directory
				TraverseDirectory(
					libInfo, sSymbolToGrep, bLookInSubFolders, bMatchCase,
					bMatchWholeWord, pDlg, bCOFF
					);
				
				// Change back to the child's parent directory
				::SetCurrentDirectory(_T(".."));
			}
			
			bOK = FindNextChildDir(hFind, &FindData);
		}

		if(hFind != INVALID_HANDLE_VALUE)
			::FindClose(hFind);
	}
}

BOOL CLibDumpView::IsChildDir(WIN32_FIND_DATA* lpFindData)
{
	return(((lpFindData->dwFileAttributes & 
			FILE_ATTRIBUTE_DIRECTORY) != 0) &&
			(lstrcmp(lpFindData->cFileName, __TEXT("."))  != 0) &&
			(lstrcmp(lpFindData->cFileName, __TEXT("..")) != 0));
}

BOOL CLibDumpView::FindNextChildDir(HANDLE hFindFile, WIN32_FIND_DATA* lpFindData)
{
	BOOL bFound = FALSE;
	do
	{
		bFound = ::FindNextFile(hFindFile, lpFindData);
	}
	while(bFound && !IsChildDir(lpFindData));

	return bFound;					
}

HANDLE CLibDumpView::FindFirstChildDir(LPTSTR lpszPattern, WIN32_FIND_DATA* lpFindData)
{
	HANDLE hFindFile = ::FindFirstFile(lpszPattern, lpFindData);
	if(hFindFile != INVALID_HANDLE_VALUE)
	{
		BOOL bFound = IsChildDir(lpFindData);
		if(!bFound)
			bFound = FindNextChildDir(hFindFile, lpFindData);
		if(!bFound)
		{
			::FindClose(hFindFile);
			hFindFile = INVALID_HANDLE_VALUE;
		}
	}
 
	return hFindFile;
}

int CLibDumpView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if(CRichEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_pEdit = &GetRichEditCtrl();
	ASSERT(m_pEdit != NULL);
	if(m_font.m_hObject != NULL)
		m_pEdit->SetFont(&m_font);
	m_pEdit->SetReadOnly();

	return 0;
}

HMENU CLibDumpView::GetContextMenu(WORD /*wSelType*/, LPOLEOBJECT /*lpOleObj*/,
	CHARRANGE* /*lpChrg*/)
{
	CMenu menu;
	if(menu.LoadMenu(IDR_RCLICK))
	{
		CMenu* pMenu = menu.GetSubMenu(0);
		ASSERT(pMenu);
		if(!m_pEdit->GetTextLength())
			pMenu->EnableMenuItem(ID_EDIT_SELECT_ALL, MF_BYCOMMAND | MF_GRAYED);

		long lStart = -1;
		long lEnd = -1;
		m_pEdit->GetSel(lStart, lEnd);
		CPoint point;
		::GetCursorPos(&point);
		ScreenToClient(&point);
		long lCharIndex = SendMessage(EM_CHARFROMPOS, 0, (long)&point);
		// Ensure that <lCharIndex> lies within the selection
		if((lEnd-lStart == 0) ||
			(lCharIndex < lStart || lCharIndex > lEnd))
			m_pEdit->SetSel(lCharIndex, lCharIndex);
		m_strFileOpen = GetFileName(lCharIndex);
		// Modify the menu string
		CString str = _T("&Open ");
		int nPos = m_strFileOpen.ReverseFind('\\');
		if(nPos != -1)
			str += m_strFileOpen.Mid(nPos+1);
		str.TrimLeft();
		str.TrimRight();
		pMenu->ModifyMenu(
			ID_RIGHTCLICK_OPEN,
			MF_BYCOMMAND,
			ID_RIGHTCLICK_OPEN,
			str
			);
		if(m_strFileOpen.IsEmpty())
			pMenu->EnableMenuItem(ID_RIGHTCLICK_OPEN, MF_BYCOMMAND | MF_GRAYED);
				
		m_pEdit->GetSel(lStart, lEnd);
		if(lEnd-lStart == 0)
			pMenu->EnableMenuItem(ID_EDIT_COPY, MF_BYCOMMAND | MF_GRAYED);
		
		ClientToScreen(&point);
		CRect rect;
		m_pEdit->GetClientRect(&rect);
		ClientToScreen(&rect);
		if(!rect.PtInRect(point))
		{
			// Shift+F10 scenario
			point = m_pEdit->GetCharPos(lCharIndex);
			ClientToScreen(&point);
		}
		pMenu->TrackPopupMenu(0, point.x, point.y, this);
	}

	return NULL;
}

void CLibDumpView::OnTextNotFound(LPCTSTR lpStr)
{
	ASSERT_VALID(this);
	CString sBuff;
	sBuff.LoadString(IDS_FINISHED_SEARCH);
	AfxMessageBox(sBuff, MB_OK | MB_ICONINFORMATION);
}

/////////////////////////////////////////////////////////////////////////////
// Dump handling

CString CLibDumpView::GetFileName(const long& lIndex)
{
	CString sFileName;

	long nCurrentLine = m_pEdit->LineFromChar(lIndex)+1;
	long lCharacterIndex = m_pEdit->LineIndex(nCurrentLine-1);
	long lLength = m_pEdit->LineLength(lCharacterIndex);
	CString sLine;
	m_pEdit->GetLine(nCurrentLine-1, sLine.GetBuffer(lLength+100));
    sLine.ReleaseBuffer(lLength+100);
	int nPos = -1;
	if(m_bCOFF)
		nPos = sLine.Find(_T(": "));
	else
		nPos = sLine.Find(_T("): "));
	if(nPos != -1)
	{
		sFileName = sLine.Left(nPos);
		if(!m_bCOFF)
		{
			nPos = sFileName.ReverseFind(_T(' '));
			if(nPos != -1)
				sFileName = sLine.Left(nPos);
		}
		CFileStatus fs;
		if(!CFile::GetStatus(sFileName, fs))
			sFileName.Empty();
	}
			
	return sFileName;
}

BOOL CLibDumpView::DumpLibrary(
	CStdioFile* pFile, const CString& sFileName, CString& sLastError
	)
{
	BOOL bRet = FALSE;

	ASSERT(pFile && pFile->m_pStream);
	sLastError.Empty();
	
	CString sBuff, sFormat;
	int nPos = sFileName.ReverseFind('\\');
	if(nPos != -1)
		sBuff = _T("..\\") + sFileName.Mid(nPos+1);
	sFormat.Format(
		_T("Library: %s\n\n"), (LPCTSTR)sBuff
		);
    pFile->WriteString(sFormat);
	
	CLibSymbolInfo libInfo;
	LPTSTR lpszPathName = (LPTSTR)(LPCTSTR)sFileName;
	try
	{
		bRet = libInfo.DumpSymbols(lpszPathName, pFile);
		if(!bRet)
			sLastError = libInfo.GetLastError();
	}
	catch(...)
	{
		sLastError = _T("Errors occurred.");
	}

	return bRet;
}

HRESULT CLibDumpView::DumpTypeLibrary(
	CStdioFile* pFile, const CString& sFileName
	)
{
	ASSERT(pFile && pFile->m_pStream);
	
	LPTYPELIB lpTypeLib = NULL;
	HRESULT hr = CTypeLib::IsTypeLib(sFileName, lpTypeLib);
	if(FAILED(hr))
		_com_issue_error(hr);
	CString sText;
	hr = CTypeLib::GetTypeLibAsString(lpTypeLib, sText);
	if(FAILED(hr))
		_com_issue_error(hr);
	pFile->WriteString(sText);
	
	return hr;
}

void CLibDumpView::OpenFile(
	CStdioFile* pFile, const CString& sFileName, const CString& sTempFile
	)
{
	CWaitCursor wait;

	m_pWndStatusBar->SetPaneText(0, _T("Please wait..."));

	BOOL bRet = FALSE;

	ASSERT(pFile && pFile->m_pStream);

	if(m_bCOFF)
	{
		CString sLastError;
		bRet = DumpLibrary(pFile, sFileName, sLastError);
		if(!bRet)
			AfxMessageBox(sLastError, MB_ICONERROR);
	}
	else
	{
		try
		{
			HRESULT hr = DumpTypeLibrary(pFile, sFileName);
			if(FAILED(hr))
				_com_issue_error(hr);

			bRet = TRUE;
		}
		catch(const _com_error& e)
		{
			AfxMessageBox(CHelpers::GetComError(e), MB_ICONERROR);
		}
		catch(...)
		{
			AfxMessageBox(_T("Errors occurred."), MB_ICONERROR);
		}
	}

	pFile->Close();

	if(bRet)
	{
		try
		{
			HINSTANCE hInstance = ::ShellExecute(
				NULL, _T("open"), sTempFile, NULL, NULL, SW_SHOWNORMAL
				);
			if((UINT)hInstance <= 32)
				_com_issue_error(HRESULT_FROM_WIN32(GetLastError()));
		}
		catch(const _com_error& e)
		{
			AfxMessageBox(CHelpers::GetComError(e), MB_ICONERROR);
		}
		catch(...)
		{
			AfxMessageBox(_T("Errors occurred."), MB_ICONERROR);
		}
	}

	CString sBuff;
	sBuff.LoadString(AFX_IDS_IDLEMESSAGE);
	m_pWndStatusBar->SetPaneText(0, sBuff);
}

void CLibDumpView::HandleFileOpen(const CString& sFileName)
{
	CString sTempPath = theApp.GetLibDumpTempPath();
	int nPos = sFileName.ReverseFind('\\');
	CString sTempFile;
	if(nPos != -1)
	{
		sTempFile = sFileName.Mid(nPos+1);
		nPos = sTempFile.ReverseFind(_T('.'));
		if(nPos != -1)
			sTempFile = sTempPath + sTempFile.Left(nPos) + _T(".txt");
	}
	CString sBuff;
	CStdioFile file;
	CFileException fileException;
	if(!file.Open(sTempFile, CFile::typeText | CFile::modeCreate |  
		CFile::modeWrite, &fileException))
	{
		sBuff.Format(_T("File Path Name: <%s> "), (LPCTSTR)sTempFile);
		sBuff += CHelpers::GetFileExceptionError(fileException.m_cause);
		AfxMessageBox(sBuff, MB_ICONERROR);
	}
	else
		OpenFile(&file, sFileName, sTempFile);
}

void CLibDumpView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	long lStartSel = -1, lEndSel = -1;
	m_pEdit->GetSel(lStartSel, lEndSel);
	CString sFileName = GetFileName(lStartSel);
	if(!sFileName.IsEmpty())
		HandleFileOpen(sFileName);
	else
		CRichEditView::OnLButtonDblClk(nFlags, point);
}

void CLibDumpView::OnRightclickOpen() 
{
	HandleFileOpen(m_strFileOpen);
}

BOOL CLibDumpView::PreTranslateMessage(MSG* pMsg) 
{
	if(pMsg->message == WM_KEYDOWN)
	{
		BOOL bCtrl = ::GetKeyState(VK_CONTROL) < 0;
		BOOL bShift = ::GetKeyState(VK_SHIFT) < 0;
		if(bCtrl && bShift && pMsg->wParam == 'L')
			return TRUE;
    }
    	
	return CRichEditView::PreTranslateMessage(pMsg);
}

/////////////////////////////////////////////////////////////////////////////
// CLibDumpView printing

BOOL CLibDumpView::OnPreparePrinting(CPrintInfo* pInfo)
{
	m_pEdit->GetSel(m_nNextSelCharToBePrint, m_nFirstNonSelChar);
	int nDiff = m_nFirstNonSelChar-m_nNextSelCharToBePrint;
	CString sText;
	if(nDiff < 20000)
		sText = m_pEdit->GetSelText();
	else
		m_pEdit->GetWindowText(sText);
	sText.TrimLeft();
	sText.TrimRight();
	if(m_nNextSelCharToBePrint != m_nFirstNonSelChar && !sText.IsEmpty())
	{
		pInfo->m_pPD->m_pd.Flags &= ~PD_NOSELECTION;
		pInfo->m_pPD->m_pd.Flags |= PD_SELECTION;
	}

	BOOL bRet = DoPreparePrinting(pInfo);
	if(bRet)
	{
		m_bPrintSelection = pInfo->m_pPD->PrintSelection();
		if(m_bPrintSelection)
			SetMargins(CRect(720, 720, 720, 720));
		else
			SetMargins(CRect(720, 720, 720, 1080));

		DWORD dwOrientation = 
			pInfo->m_pPD->GetDevMode()->dmOrientation;
		CSize csPaper = GetPaperSize();
		if(dwOrientation == DMORIENT_LANDSCAPE)
		{
			if(csPaper.cx < csPaper.cy)
			{
				int nTemp = csPaper.cx; csPaper.cx = csPaper.cy; csPaper.cy = nTemp;
				SetPaperSize(csPaper);
			}
		}
		else
		{
			if(csPaper.cx > csPaper.cy)
			{
				int nTemp = csPaper.cx; csPaper.cx = csPaper.cy; csPaper.cy = nTemp;
				SetPaperSize(csPaper);
			}
		}
	}
		
	return bRet;
}

void CLibDumpView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(lf));
	VERIFY(m_font.GetLogFont(&lf));
	// Map to printer font metrics
	HDC hDCFrom = ::GetDC(NULL);
	lf.lfHeight = ::MulDiv(lf.lfHeight, pDC->GetDeviceCaps(LOGPIXELSY),
		::GetDeviceCaps(hDCFrom, LOGPIXELSY));
	lf.lfWidth = ::MulDiv(lf.lfWidth, pDC->GetDeviceCaps(LOGPIXELSX),
		::GetDeviceCaps(hDCFrom, LOGPIXELSX));
	::ReleaseDC(NULL, hDCFrom);
	VERIFY(m_fontPrinter.CreateFontIndirect(&lf));

	CRichEditView::OnBeginPrinting(pDC, pInfo);

	if(!m_bPrintSelection)
	{
		// Get text width
		TEXTMETRIC tm;
		memset(&tm, 0, sizeof(tm));
		CDC* pDCCtrl = GetRichEditCtrl().GetDC();
		if(pDCCtrl == NULL)
			return;
		VERIFY(pDCCtrl->GetTextMetrics(&tm));

		// Find rows per page
		double d =
			(double)pDC->GetDeviceCaps(LOGPIXELSY)/(double)pDCCtrl->GetDeviceCaps(LOGPIXELSY);
		int nPageHeight = pDC->GetDeviceCaps(VERTRES);
		int nRowsPerPage =
			(int)((double)nPageHeight/d)/tm.tmHeight-2*3.33;
		
		int nWordWrap = m_nWordWrap;
		m_nWordWrap = WrapToTargetDevice;
		WrapChanged();
		pInfo->SetMaxPage(
			(GetRichEditCtrl().GetLineCount()-1)/nRowsPerPage+1
			);
		m_nWordWrap = nWordWrap;
		WrapChanged();
		GetRichEditCtrl().ReleaseDC(pDCCtrl);
	}
}

void CLibDumpView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	CRichEditView::OnEndPrinting(pDC, pInfo);

	if(m_fontPrinter.m_hObject != NULL)
	{
		m_fontPrinter.DeleteObject();
		m_fontPrinter.m_hObject = NULL;
	}
}

void CLibDumpView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if(!m_bPrintSelection)
		CRichEditView::OnPrepareDC(pDC, pInfo);
	else
		pInfo->m_bContinuePrinting = m_nNextSelCharToBePrint < m_nFirstNonSelChar;
}

void CLibDumpView::OnFilePrint() 
{
	CWaitCursor wait;

	CDocument* pDoc = GetDocument();
	ASSERT(pDoc != NULL);
	if(m_strLibName.IsEmpty())
		m_strLibName = _T("Untitled");
	pDoc->SetTitle(m_strLibName);
	CRichEditView::OnFilePrint();	
}

static void MulDivRect(LPRECT r1, LPRECT r2, int num, int div)
{
	r1->left = ::MulDiv(r2->left, num, div);
	r1->top = ::MulDiv(r2->top, num, div);
	r1->right = ::MulDiv(r2->right, num, div);
	r1->bottom = ::MulDiv(r2->bottom, num, div);
}

static void GetRectInTwips(CDC* pDC, CRect& rect)
{
	ASSERT(pDC != NULL);

	int logx = ::GetDeviceCaps(pDC->m_hDC, LOGPIXELSX);
	int logy = ::GetDeviceCaps(pDC->m_hDC, LOGPIXELSY);
	rect.left = ::MulDiv(rect.left, logx, 1440);
	rect.right = ::MulDiv(rect.right, logx, 1440);
	rect.top = ::MulDiv(rect.top, logy, 1440);
	rect.bottom = ::MulDiv(rect.bottom, logy, 1440);
}

void CLibDumpView::OnPrint(CDC* pDC, CPrintInfo* pInfo)
{
	if(m_bPrintSelection)
	{
		CRect rect;
		rect.left = m_rectMargin.left;
		rect.right = m_sizePaper.cx - m_rectMargin.right;
		rect.top = m_rectMargin.top;
		rect.bottom = m_sizePaper.cy - m_rectMargin.bottom;
		m_nNextSelCharToBePrint = PrintInsideRect(
			pDC,
			rect,
			m_nNextSelCharToBePrint,
			m_nFirstNonSelChar,
			TRUE
			);
	}
	else
	{
		CRichEditView::OnPrint(pDC, pInfo);

		if(pInfo != NULL && pDC->m_hAttribDC != NULL)
		{
			// Footer
			CFont* pOldFont = pDC->SelectObject(&m_fontPrinter);
			CRect rect;
			rect.left = m_rectMargin.left;
			rect.right = m_sizePaper.cx - m_rectMargin.right;
			rect.top = m_sizePaper.cy - m_rectMargin.bottom;
			rect.bottom = rect.top + 3*m_rectMargin.bottom/4;
			GetRectInTwips(pDC, rect);
			COleDateTime dt(COleDateTime::GetCurrentTime());
			CString sBuff = dt.Format();
			pDC->DrawText(
				sBuff, rect, DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER
				);
			sBuff.Format(_T("Page %d"), pInfo->m_nCurPage);
			pDC->DrawText(
				sBuff, rect, DT_RIGHT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER
				);
			pDC->SelectObject(pOldFont);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////