// LibDumpView.h : interface of the CLibDumpView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_LIBDUMPVIEW_H__94ED91AD_B7B6_11D1_BA22_204C4F4F5020__INCLUDED_)
#define AFX_LIBDUMPVIEW_H__94ED91AD_B7B6_11D1_BA22_204C4F4F5020__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "LibDumpDoc.h"
#include "LibSymbolInfo.h"
#include "StopDlg.h"

class CLibDumpView : public CRichEditView
{
protected: // create from serialization only
	CLibDumpView();
	DECLARE_DYNCREATE(CLibDumpView)
	CStatusBar* m_pWndStatusBar;

// Attributes
public:
	CLibDumpDoc* GetDocument();
	BOOL DumpLibrary(
		CStdioFile* pFile, const CString& sFileName, CString& sLastError
		);
	HRESULT DumpTypeLibrary(CStdioFile* pFile, const CString& sFileName);
	
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CLibDumpView)
	public:
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void OnInitialUpdate(); // called first time after construct
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CLibDumpView();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	virtual HMENU GetContextMenu(
		WORD wSelType, LPOLEOBJECT lpOleObj, CHARRANGE* lpChrg
		);
	virtual void OnTextNotFound(LPCTSTR);

// Generated message map functions
protected:
	//{{AFX_MSG(CLibDumpView)
	afx_msg void OnFileDump();
	afx_msg void OnFileFindSymbols();
	afx_msg void OnFilePrint();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnFileSaveAs();
	afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
	afx_msg void OnEditWordWrap();
	afx_msg void OnUpdateEditWordWrap(CCmdUI* pCmdUI);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRightclickOpen();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:
	CRichEditCtrl* m_pEdit;
	CString m_strFileOpen;
	void HandleFileOpen(const CString& sFileName);
	inline bool _istpunctEx(const TCHAR& ch)
	{
		return (
			ch == _T(' ') ||
			ch == _T('\t') ||
			ch == _T('\r') ||
			ch == _T('\n') ||
			_istpunct(ch)
			);
	}
	inline bool MatchWholeWord(
		const int& nFind,
		const CString& sSymbolInfo,
		const CString& sSymbolToGrep
		)
	{
		bool bRet = false;

		int nSymbolInfoLen = sSymbolInfo.GetLength();
		int nSymbolToGrepLen = sSymbolToGrep.GetLength();
		if(nFind == 0 && ((nSymbolInfoLen-1) == nSymbolToGrepLen))
		{
			bRet = true;
		}
		else if(nFind > 0 && ((nFind+nSymbolToGrepLen) < nSymbolInfoLen))
		{
			bRet = (
				_istpunctEx(sSymbolInfo.GetAt(nFind-1)) &&
				_istpunctEx(sSymbolInfo.GetAt(nFind+nSymbolToGrepLen))
				);
		}
		
		return bRet;
	}
	void GrepTypeLibSymbols(
		const CString& sLibFilePath, const CString& sSymbolToGrep,
		const BOOL& bMatchCase, const BOOL& bMatchWholeWord, CStopDlg* pDlg
		);
	void TraverseDirectory(CLibSymbolInfo& libInfo, const CString& sSymbolToGrep,
		const BOOL& bLookInSubFolders, const BOOL& bMatchCase, const BOOL& bMatchWholeWord,
		CStopDlg* pDlg, const BOOL& bCOFF = TRUE);
	HANDLE FindFirstChildDir(LPTSTR lpszPattern, WIN32_FIND_DATA* lpFindData);
	BOOL FindNextChildDir(HANDLE hFindFile, WIN32_FIND_DATA* lpFindData);
	BOOL IsChildDir(WIN32_FIND_DATA* lpFindData);
	BOOL m_bCOFF;
	void OpenFile(
		CStdioFile* pFile, const CString& sFileName, const CString& sTempFile
		);
	CString GetFileName(const long& lIndex);
	CString m_strLibName;
	BOOL m_bPrintSelection;
	long m_nNextSelCharToBePrint;
	long m_nFirstNonSelChar;
	CFont m_font, m_fontPrinter;
	long m_nTypeLibSymbolOccurences;
};

#ifndef _DEBUG  // debug version in LibDumpView.cpp
inline CLibDumpDoc* CLibDumpView::GetDocument()
   { return (CLibDumpDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_LIBDUMPVIEW_H__94ED91AD_B7B6_11D1_BA22_204C4F4F5020__INCLUDED_)