// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "LibDump.h"
#include "MainFrm.h"
#include "LibDumpView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_COMMAND(ID_HELP_README, OnHelpReadme)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_POSITION, OnUpdateIndicatorPosition)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_POSITION,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame() :
	m_nLineColPaneSize(100)
{
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if(!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE(_T("Failed to create toolbar\n"));
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(
		m_wndToolBar.GetBarStyle() | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC
		);
	m_wndToolBar.SetHeight(30);

	UINT nID = ID_SEPARATOR;
	UINT nStyle = -1;
	m_nLineColPaneSize = CHelpers::GetPaneSize(
		1,
		nID,
		nStyle,
		&m_wndStatusBar,
		_T("Ln 8888, Col 8888")
		);
	m_wndStatusBar.SetPaneInfo(
		1, ID_INDICATOR_POSITION, SBPS_NORMAL, m_nLineColPaneSize 
		);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style = WS_OVERLAPPED | WS_CAPTION /*| FWS_ADDTOTITLE*/
		| WS_THICKFRAME | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_MAXIMIZE;

	return CFrameWnd::PreCreateWindow(cs);
}

void CMainFrame::OnUpdateIndicatorPosition(CCmdUI* pCmdUI)
{
	CRichEditCtrl* pRichEditCtrl =
		&((CLibDumpView*)GetActiveView())->GetRichEditCtrl();
	if(pRichEditCtrl != NULL)
	{
		long lStart = -1;
		long lEnd = -1;
		pRichEditCtrl->GetSel(lStart, lEnd);
		m_strLineColInfo.Format(
			_T("Ln %d, Col %d"),
			pRichEditCtrl->LineFromChar(lStart)+1,
			((lEnd-lStart) >= 0 ? lEnd-pRichEditCtrl->LineIndex(-1) :
				lStart-pRichEditCtrl->LineIndex(-1))+1
			);
		UINT nID = 0, nStyle = 0;
		int nSize = CHelpers::GetPaneSize(
			1, nID, nStyle, &m_wndStatusBar, m_strLineColInfo
			);
		if(nSize > m_nLineColPaneSize)
			m_wndStatusBar.SetPaneInfo(1, nID, nStyle, nSize);
		pCmdUI->SetText(m_strLineColInfo);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Helpers for saving/restoring window state

static TCHAR BASED_CODE szSection[] = _T("Settings");
static TCHAR BASED_CODE szWindowPos[] = _T("WindowPos");
static TCHAR szFormat[] = _T("%u,%u,%d,%d,%d,%d,%d,%d,%d,%d");

static BOOL PASCAL ReadWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	CWinApp* pApp = AfxGetApp();
	CString strBuffer = pApp->GetProfileString(szSection, szWindowPos);
	if(strBuffer.IsEmpty())
		return FALSE;

	WINDOWPLACEMENT wp;
	int nRead = _stscanf(strBuffer, szFormat,
		&wp.flags, &wp.showCmd,
		&wp.ptMinPosition.x, &wp.ptMinPosition.y,
		&wp.ptMaxPosition.x, &wp.ptMaxPosition.y,
		&wp.rcNormalPosition.left, &wp.rcNormalPosition.top,
		&wp.rcNormalPosition.right, &wp.rcNormalPosition.bottom);

	if(nRead != 10)
		return FALSE;

	wp.length = sizeof(wp);
	*pwp = wp;

	return TRUE;
}

static void PASCAL WriteWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	TCHAR szBuffer[sizeof(_T("-32767"))*8 + sizeof(_T("65535"))*2];

	wsprintf(szBuffer, szFormat,
		pwp->flags, pwp->showCmd,
		pwp->ptMinPosition.x, pwp->ptMinPosition.y,
		pwp->ptMaxPosition.x, pwp->ptMaxPosition.y,
		pwp->rcNormalPosition.left, pwp->rcNormalPosition.top,
		pwp->rcNormalPosition.right, pwp->rcNormalPosition.bottom);
	
	AfxGetApp()->WriteProfileString(szSection, szWindowPos, szBuffer);
}

/////////////////////////////////////////////////////////////////////////////

void CMainFrame::InitialShowWindow(UINT nCmdShow)
{
	WINDOWPLACEMENT wp;
	if(!ReadWindowPlacement(&wp))
		CenterWindow();
	else
	{
		if(nCmdShow != SW_SHOWNORMAL)
			wp.showCmd = nCmdShow;
		else
			nCmdShow = wp.showCmd;

		SetWindowPlacement(&wp);
	}
	
	ShowWindow(nCmdShow);
	UpdateWindow();
}

void CMainFrame::OnClose() 
{
	WINDOWPLACEMENT wp;
	wp.length = sizeof(wp);
	if(GetWindowPlacement(&wp))
	{
		wp.flags = 0;
	
		if(IsZoomed())
			wp.flags |= WPF_RESTORETOMAXIMIZED;
		
		if(wp.showCmd != SW_SHOWMINIMIZED)
			WriteWindowPlacement(&wp);
	}
	
	try
	{
		GetActiveDocument()->SetModifiedFlag(FALSE);
		CFrameWnd::OnClose();
	}
	catch(...)
	{
	}
}

void CMainFrame::OnHelpReadme() 
{
	TCHAR lpszModuleFileName[_MAX_PATH*2];
	DWORD dwSize = ::GetModuleFileName(
				AfxGetInstanceHandle(), lpszModuleFileName, _MAX_PATH*2
				);
	ASSERT(dwSize != 0);
	CString sFileName = lpszModuleFileName;
	int nPos = sFileName.ReverseFind('\\');
	if(nPos != -1)
		sFileName = sFileName.Left(nPos) + _T("\\readme.txt");
	
	try
	{
		HINSTANCE hInstance = ::ShellExecute(
			NULL, _T("open"), sFileName, NULL, NULL, SW_SHOWNORMAL
			);
		if((UINT)hInstance <= 32)
			_com_issue_error(HRESULT_FROM_WIN32(GetLastError()));
	}
	catch(const _com_error& e)
	{
		AfxMessageBox(CHelpers::GetComError(e), MB_ICONERROR);
	}
	catch(...)
	{
		AfxMessageBox(_T("Errors occurred."), MB_ICONERROR);
	}
}